/*
 * CairoContext.java
 *
 * Copyright (c) 2006-2009 Operational Dynamics Consulting Pty Ltd, and Others
 *
 * The code in this file, and the program it is a part of, are made available
 * to you by the authors under the terms of the "GNU General Public Licence,
 * version 2" plus the "Classpath Exception" (you may link to this code as a
 * library into other programs provided you don't make a derivation of it).
 * See the LICENCE file for the terms governing usage and redistribution.
 *
 *                      THIS FILE IS GENERATED CODE!
 *
 * To modify its contents or behaviour, either update the generation program,
 * change the information in the source defs file, or implement an override for
 * this class.
 */
package org.freedesktop.cairo;

import org.freedesktop.cairo.Antialias;
import org.freedesktop.cairo.Context;
import org.freedesktop.cairo.Matrix;
import org.freedesktop.cairo.Operator;
import org.freedesktop.cairo.Pattern;
import org.freedesktop.cairo.Plumbing;
import org.freedesktop.cairo.Status;
import org.freedesktop.cairo.Surface;
import org.gnome.pango.Layout;
import org.gnome.pango.LayoutLine;

final class CairoContext extends Plumbing
{
    private CairoContext() {}

    static final long createContext(Surface target) {
        long result;

        if (target == null) {
            throw new IllegalArgumentException("target can't be null");
        }

        synchronized (lock) {
            result = cairo_create(pointerOf(target));

            return result;
        }
    }

    private static native final long cairo_create(long target);

    static final void destroy(Context self) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_destroy(pointerOf(self));
        }
    }

    private static native final void cairo_destroy(long self);

    static final void save(Context self) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_save(pointerOf(self));
        }
    }

    private static native final void cairo_save(long self);

    static final void restore(Context self) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_restore(pointerOf(self));
        }
    }

    private static native final void cairo_restore(long self);

    static final void clip(Context self) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_clip(pointerOf(self));
        }
    }

    private static native final void cairo_clip(long self);

    static final void clipPreserve(Context self) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_clip_preserve(pointerOf(self));
        }
    }

    private static native final void cairo_clip_preserve(long self);

    static final void stroke(Context self) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_stroke(pointerOf(self));
        }
    }

    private static native final void cairo_stroke(long self);

    static final void strokePreserve(Context self) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_stroke_preserve(pointerOf(self));
        }
    }

    private static native final void cairo_stroke_preserve(long self);

    static final void rotate(Context self, double rad) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_rotate(pointerOf(self), rad);
        }
    }

    private static native final void cairo_rotate(long self, double rad);

    static final void translate(Context self, double tx, double ty) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_translate(pointerOf(self), tx, ty);
        }
    }

    private static native final void cairo_translate(long self, double tx, double ty);

    static final void scale(Context self, double sx, double sy) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_scale(pointerOf(self), sx, sy);
        }
    }

    private static native final void cairo_scale(long self, double sx, double sy);

    static final void setSourceRgb(Context self, double red, double green, double blue) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_set_source_rgb(pointerOf(self), red, green, blue);
        }
    }

    private static native final void cairo_set_source_rgb(long self, double red, double green, double blue);

    static final void setSourceRgba(Context self, double red, double green, double blue, double alpha) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_set_source_rgba(pointerOf(self), red, green, blue, alpha);
        }
    }

    private static native final void cairo_set_source_rgba(long self, double red, double green, double blue, double alpha);

    static final void lineTo(Context self, double x, double y) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_line_to(pointerOf(self), x, y);
        }
    }

    private static native final void cairo_line_to(long self, double x, double y);

    static final void relLineTo(Context self, double x, double y) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_rel_line_to(pointerOf(self), x, y);
        }
    }

    private static native final void cairo_rel_line_to(long self, double x, double y);

    static final void setLineWidth(Context self, double width) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_set_line_width(pointerOf(self), width);
        }
    }

    private static native final void cairo_set_line_width(long self, double width);

    static final void setAntialias(Context self, Antialias antialias) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (antialias == null) {
            throw new IllegalArgumentException("antialias can't be null");
        }

        synchronized (lock) {
            cairo_set_antialias(pointerOf(self), numOf(antialias));
        }
    }

    private static native final void cairo_set_antialias(long self, int antialias);

    static final void moveTo(Context self, double x, double y) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_move_to(pointerOf(self), x, y);
        }
    }

    private static native final void cairo_move_to(long self, double x, double y);

    static final void relMoveTo(Context self, double x, double y) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_rel_move_to(pointerOf(self), x, y);
        }
    }

    private static native final void cairo_rel_move_to(long self, double x, double y);

    static final void rectangle(Context self, double x, double y, double width, double height) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_rectangle(pointerOf(self), x, y, width, height);
        }
    }

    private static native final void cairo_rectangle(long self, double x, double y, double width, double height);

    static final void arc(Context self, double xc, double yc, double radius, double angle1, double angle2) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_arc(pointerOf(self), xc, yc, radius, angle1, angle2);
        }
    }

    private static native final void cairo_arc(long self, double xc, double yc, double radius, double angle1, double angle2);

    static final void arcNegative(Context self, double xc, double yc, double radius, double angle1, double angle2) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_arc_negative(pointerOf(self), xc, yc, radius, angle1, angle2);
        }
    }

    private static native final void cairo_arc_negative(long self, double xc, double yc, double radius, double angle1, double angle2);

    static final void fill(Context self) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_fill(pointerOf(self));
        }
    }

    private static native final void cairo_fill(long self);

    static final void fillPreserve(Context self) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_fill_preserve(pointerOf(self));
        }
    }

    private static native final void cairo_fill_preserve(long self);

    static final void showLayout(Context self, Layout layout) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (layout == null) {
            throw new IllegalArgumentException("layout can't be null");
        }

        synchronized (lock) {
            pango_cairo_show_layout(pointerOf(self), pointerOf(layout));
        }
    }

    private static native final void pango_cairo_show_layout(long self, long layout);

    static final void updateLayout(Context self, Layout layout) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (layout == null) {
            throw new IllegalArgumentException("layout can't be null");
        }

        synchronized (lock) {
            pango_cairo_update_layout(pointerOf(self), pointerOf(layout));
        }
    }

    private static native final void pango_cairo_update_layout(long self, long layout);

    static final void showLayoutLine(Context self, LayoutLine line) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (line == null) {
            throw new IllegalArgumentException("line can't be null");
        }

        synchronized (lock) {
            pango_cairo_show_layout_line(pointerOf(self), pointerOf(line));
        }
    }

    private static native final void pango_cairo_show_layout_line(long self, long line);

    static final Pattern getSource(Context self) {
        long result;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            result = cairo_get_source(pointerOf(self));

            return (Pattern) entityFor(Pattern.class,  result);
        }
    }

    private static native final long cairo_get_source(long self);

    static final Status status(Context self) {
        int result;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            result = cairo_status(pointerOf(self));

            return (Status) enumFor(Status.class, result);
        }
    }

    private static native final int cairo_status(long self);

    static final Surface getTarget(Context self) {
        long result;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            result = cairo_get_target(pointerOf(self));

            return (Surface) entityFor(Surface.class,  result);
        }
    }

    private static native final long cairo_get_target(long self);

    static final void setOperator(Context self, Operator op) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (op == null) {
            throw new IllegalArgumentException("op can't be null");
        }

        synchronized (lock) {
            cairo_set_operator(pointerOf(self), numOf(op));
        }
    }

    private static native final void cairo_set_operator(long self, int op);

    static final void paint(Context self) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_paint(pointerOf(self));
        }
    }

    private static native final void cairo_paint(long self);

    static final void paintWithAlpha(Context self, double alpha) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_paint_with_alpha(pointerOf(self), alpha);
        }
    }

    private static native final void cairo_paint_with_alpha(long self, double alpha);

    static final void setSource(Context self, Pattern pattern) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (pattern == null) {
            throw new IllegalArgumentException("pattern can't be null");
        }

        synchronized (lock) {
            cairo_set_source(pointerOf(self), pointerOf(pattern));
        }
    }

    private static native final void cairo_set_source(long self, long pattern);

    static final void mask(Context self, Pattern pattern) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (pattern == null) {
            throw new IllegalArgumentException("pattern can't be null");
        }

        synchronized (lock) {
            cairo_mask(pointerOf(self), pointerOf(pattern));
        }
    }

    private static native final void cairo_mask(long self, long pattern);

    static final void maskSurface(Context self, Surface surface, double x, double y) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (surface == null) {
            throw new IllegalArgumentException("surface can't be null");
        }

        synchronized (lock) {
            cairo_mask_surface(pointerOf(self), pointerOf(surface), x, y);
        }
    }

    private static native final void cairo_mask_surface(long self, long surface, double x, double y);

    static final void setSourceSurface(Context self, Surface surface, double x, double y) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (surface == null) {
            throw new IllegalArgumentException("surface can't be null");
        }

        synchronized (lock) {
            cairo_set_source_surface(pointerOf(self), pointerOf(surface), x, y);
        }
    }

    private static native final void cairo_set_source_surface(long self, long surface, double x, double y);

    static final void getCurrentPoint(Context self, double[] x, double[] y) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (x == null) {
            throw new IllegalArgumentException("x can't be null");
        }

        if (y == null) {
            throw new IllegalArgumentException("y can't be null");
        }

        synchronized (lock) {
            cairo_get_current_point(pointerOf(self), x, y);
        }
    }

    private static native final void cairo_get_current_point(long self, double[] x, double[] y);

    static final String statusToString(Status status) {
        String result;

        if (status == null) {
            throw new IllegalArgumentException("status can't be null");
        }

        synchronized (lock) {
            result = cairo_status_to_string(numOf(status));

            return result;
        }
    }

    private static native final String cairo_status_to_string(int status);

    static final void transform(Context self, Matrix matrix) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (matrix == null) {
            throw new IllegalArgumentException("matrix can't be null");
        }

        synchronized (lock) {
            cairo_transform(pointerOf(self), pointerOf(matrix));
        }
    }

    private static native final void cairo_transform(long self, long matrix);
}

