/*
 * CairoPattern.java
 *
 * Copyright (c) 2006-2009 Operational Dynamics Consulting Pty Ltd, and Others
 *
 * The code in this file, and the program it is a part of, are made available
 * to you by the authors under the terms of the "GNU General Public Licence,
 * version 2" plus the "Classpath Exception" (you may link to this code as a
 * library into other programs provided you don't make a derivation of it).
 * See the LICENCE file for the terms governing usage and redistribution.
 *
 *                      THIS FILE IS GENERATED CODE!
 *
 * To modify its contents or behaviour, either update the generation program,
 * change the information in the source defs file, or implement an override for
 * this class.
 */
package org.freedesktop.cairo;

import org.freedesktop.cairo.Pattern;
import org.freedesktop.cairo.Plumbing;
import org.freedesktop.cairo.Status;
import org.freedesktop.cairo.Surface;

final class CairoPattern extends Plumbing
{
    private CairoPattern() {}

    static final void destroy(Pattern self) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_pattern_destroy(pointerOf(self));
        }
    }

    private static native final void cairo_pattern_destroy(long self);

    static final Pattern reference(Pattern self) {
        long result;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            result = cairo_pattern_reference(pointerOf(self));

            return (Pattern) entityFor(Pattern.class,  result);
        }
    }

    private static native final long cairo_pattern_reference(long self);

    static final long createPatternForSurface(Surface surface) {
        long result;

        if (surface == null) {
            throw new IllegalArgumentException("surface can't be null");
        }

        synchronized (lock) {
            result = cairo_pattern_create_for_surface(pointerOf(surface));

            return result;
        }
    }

    private static native final long cairo_pattern_create_for_surface(long surface);

    static final long createPatternRgba(double red, double green, double blue, double alpha) {
        long result;

        synchronized (lock) {
            result = cairo_pattern_create_rgba(red, green, blue, alpha);

            return result;
        }
    }

    private static native final long cairo_pattern_create_rgba(double red, double green, double blue, double alpha);

    static final long createPatternLinear(double x0, double y0, double x1, double y1) {
        long result;

        synchronized (lock) {
            result = cairo_pattern_create_linear(x0, y0, x1, y1);

            return result;
        }
    }

    private static native final long cairo_pattern_create_linear(double x0, double y0, double x1, double y1);

    static final long createPatternRadial(double cx0, double cy0, double radius0, double cx1, double cy1, double radius1) {
        long result;

        synchronized (lock) {
            result = cairo_pattern_create_radial(cx0, cy0, radius0, cx1, cy1, radius1);

            return result;
        }
    }

    private static native final long cairo_pattern_create_radial(double cx0, double cy0, double radius0, double cx1, double cy1, double radius1);

    static final void addColorStopRgb(Pattern self, double offset, double red, double green, double blue) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_pattern_add_color_stop_rgb(pointerOf(self), offset, red, green, blue);
        }
    }

    private static native final void cairo_pattern_add_color_stop_rgb(long self, double offset, double red, double green, double blue);

    static final void addColorStopRgba(Pattern self, double offset, double red, double green, double blue, double alpha) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            cairo_pattern_add_color_stop_rgba(pointerOf(self), offset, red, green, blue, alpha);
        }
    }

    private static native final void cairo_pattern_add_color_stop_rgba(long self, double offset, double red, double green, double blue, double alpha);

    static final Status status(Pattern self) {
        int result;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            result = cairo_pattern_status(pointerOf(self));

            return (Status) enumFor(Status.class, result);
        }
    }

    private static native final int cairo_pattern_status(long self);
}

