/*
 * AtkStateSet.java
 *
 * Copyright (c) 2006-2009 Operational Dynamics Consulting Pty Ltd, and Others
 *
 * The code in this file, and the program it is a part of, are made available
 * to you by the authors under the terms of the "GNU General Public Licence,
 * version 2" plus the "Classpath Exception" (you may link to this code as a
 * library into other programs provided you don't make a derivation of it).
 * See the LICENCE file for the terms governing usage and redistribution.
 *
 *                      THIS FILE IS GENERATED CODE!
 *
 * To modify its contents or behaviour, either update the generation program,
 * change the information in the source defs file, or implement an override for
 * this class.
 */
package org.gnome.atk;

import org.gnome.atk.Plumbing;
import org.gnome.atk.StateSet;
import org.gnome.atk.StateType;

final class AtkStateSet extends Plumbing
{
    private AtkStateSet() {}

    static final long createStateSet() {
        long result;

        synchronized (lock) {
            result = atk_state_set_new();

            return result;
        }
    }

    private static native final long atk_state_set_new();

    static final boolean isEmpty(StateSet self) {
        boolean result;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            result = atk_state_set_is_empty(pointerOf(self));

            return result;
        }
    }

    private static native final boolean atk_state_set_is_empty(long self);

    static final boolean addState(StateSet self, StateType type) {
        boolean result;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (type == null) {
            throw new IllegalArgumentException("type can't be null");
        }

        synchronized (lock) {
            result = atk_state_set_add_state(pointerOf(self), numOf(type));

            return result;
        }
    }

    private static native final boolean atk_state_set_add_state(long self, int type);

    static final void addStates(StateSet self, StateType[] types, int nTypes) {
        int[] _types;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (types == null) {
            throw new IllegalArgumentException("types can't be null");
        }

        _types = numsOf(types);

        synchronized (lock) {
            atk_state_set_add_states(pointerOf(self), _types, nTypes);
            fillEnumArray(StateType.class, types, _types);
        }
    }

    private static native final void atk_state_set_add_states(long self, int[] types, int nTypes);

    static final void clearStates(StateSet self) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            atk_state_set_clear_states(pointerOf(self));
        }
    }

    private static native final void atk_state_set_clear_states(long self);

    static final boolean containsState(StateSet self, StateType type) {
        boolean result;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (type == null) {
            throw new IllegalArgumentException("type can't be null");
        }

        synchronized (lock) {
            result = atk_state_set_contains_state(pointerOf(self), numOf(type));

            return result;
        }
    }

    private static native final boolean atk_state_set_contains_state(long self, int type);

    static final boolean containsStates(StateSet self, StateType[] types, int nTypes) {
        boolean result;
        int[] _types;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (types == null) {
            throw new IllegalArgumentException("types can't be null");
        }

        _types = numsOf(types);

        synchronized (lock) {
            result = atk_state_set_contains_states(pointerOf(self), _types, nTypes);
            fillEnumArray(StateType.class, types, _types);

            return result;
        }
    }

    private static native final boolean atk_state_set_contains_states(long self, int[] types, int nTypes);

    static final boolean removeState(StateSet self, StateType type) {
        boolean result;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (type == null) {
            throw new IllegalArgumentException("type can't be null");
        }

        synchronized (lock) {
            result = atk_state_set_remove_state(pointerOf(self), numOf(type));

            return result;
        }
    }

    private static native final boolean atk_state_set_remove_state(long self, int type);

    static final StateSet andSets(StateSet self, StateSet compareSet) {
        long result;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (compareSet == null) {
            throw new IllegalArgumentException("compareSet can't be null");
        }

        synchronized (lock) {
            result = atk_state_set_and_sets(pointerOf(self), pointerOf(compareSet));

            return (StateSet) objectFor(result);
        }
    }

    private static native final long atk_state_set_and_sets(long self, long compareSet);

    static final StateSet orSets(StateSet self, StateSet compareSet) {
        long result;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (compareSet == null) {
            throw new IllegalArgumentException("compareSet can't be null");
        }

        synchronized (lock) {
            result = atk_state_set_or_sets(pointerOf(self), pointerOf(compareSet));

            return (StateSet) objectFor(result);
        }
    }

    private static native final long atk_state_set_or_sets(long self, long compareSet);

    static final StateSet xorSets(StateSet self, StateSet compareSet) {
        long result;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (compareSet == null) {
            throw new IllegalArgumentException("compareSet can't be null");
        }

        synchronized (lock) {
            result = atk_state_set_xor_sets(pointerOf(self), pointerOf(compareSet));

            return (StateSet) objectFor(result);
        }
    }

    private static native final long atk_state_set_xor_sets(long self, long compareSet);
}

