/*
 * GdkDragContext.java
 *
 * Copyright (c) 2006-2009 Operational Dynamics Consulting Pty Ltd, and Others
 *
 * The code in this file, and the program it is a part of, are made available
 * to you by the authors under the terms of the "GNU General Public Licence,
 * version 2" plus the "Classpath Exception" (you may link to this code as a
 * library into other programs provided you don't make a derivation of it).
 * See the LICENCE file for the terms governing usage and redistribution.
 *
 *                      THIS FILE IS GENERATED CODE!
 *
 * To modify its contents or behaviour, either update the generation program,
 * change the information in the source defs file, or implement an override for
 * this class.
 */
package org.gnome.gdk;

import org.freedesktop.bindings.BlacklistedMethodError;
import org.freedesktop.bindings.FIXME;
import org.gnome.gdk.Bitmap;
import org.gnome.gdk.Colormap;
import org.gnome.gdk.DragAction;
import org.gnome.gdk.DragContext;
import org.gnome.gdk.DragProtocol;
import org.gnome.gdk.Pixbuf;
import org.gnome.gdk.Pixmap;
import org.gnome.gdk.Plumbing;
import org.gnome.gdk.Screen;
import org.gnome.gdk.Window;
import org.gnome.gtk.Widget;

final class GdkDragContext extends Plumbing
{
    private GdkDragContext() {}

    static final long createDragContext() {
        long result;

        synchronized (lock) {
            result = gdk_drag_context_new();

            return result;
        }
    }

    private static native final long gdk_drag_context_new();

    static final void dragStatus(DragContext self, DragAction action, int time) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (action == null) {
            throw new IllegalArgumentException("action can't be null");
        }

        synchronized (lock) {
            gdk_drag_status(pointerOf(self), numOf(action), time);
        }
    }

    private static native final void gdk_drag_status(long self, int action, int time);

    static final void dropReply(DragContext self, boolean ok, int time) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            gdk_drop_reply(pointerOf(self), ok, time);
        }
    }

    private static native final void gdk_drop_reply(long self, boolean ok, int time);

    static final void dropFinish(DragContext self, boolean success, int time) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            gdk_drop_finish(pointerOf(self), success, time);
        }
    }

    private static native final void gdk_drop_finish(long self, boolean success, int time);

    static final FIXME dragGetSelection(DragContext self) throws BlacklistedMethodError {
        throw new BlacklistedMethodError("GdkAtom");
    }

    static final void dragFindWindow(DragContext self, Window dragWindow, int xRoot, int yRoot, Window[] destWindow, DragProtocol[] protocol) {
        long[] _destWindow;
        int[] _protocol;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (dragWindow == null) {
            throw new IllegalArgumentException("dragWindow can't be null");
        }

        if (destWindow == null) {
            throw new IllegalArgumentException("destWindow can't be null");
        }

        if (protocol == null) {
            throw new IllegalArgumentException("protocol can't be null");
        }

        _destWindow = pointersOf(destWindow);

        _protocol = numsOf(protocol);

        synchronized (lock) {
            gdk_drag_find_window(pointerOf(self), pointerOf(dragWindow), xRoot, yRoot, _destWindow, _protocol);
            fillObjectArray(destWindow, _destWindow);
            fillEnumArray(DragProtocol.class, protocol, _protocol);
        }
    }

    private static native final void gdk_drag_find_window(long self, long dragWindow, int xRoot, int yRoot, long[] destWindow, int[] protocol);

    static final void dragFindWindowForScreen(DragContext self, Window dragWindow, Screen screen, int xRoot, int yRoot, Window[] destWindow, DragProtocol[] protocol) {
        long[] _destWindow;
        int[] _protocol;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (dragWindow == null) {
            throw new IllegalArgumentException("dragWindow can't be null");
        }

        if (screen == null) {
            throw new IllegalArgumentException("screen can't be null");
        }

        if (destWindow == null) {
            throw new IllegalArgumentException("destWindow can't be null");
        }

        if (protocol == null) {
            throw new IllegalArgumentException("protocol can't be null");
        }

        _destWindow = pointersOf(destWindow);

        _protocol = numsOf(protocol);

        synchronized (lock) {
            gdk_drag_find_window_for_screen(pointerOf(self), pointerOf(dragWindow), pointerOf(screen), xRoot, yRoot, _destWindow, _protocol);
            fillObjectArray(destWindow, _destWindow);
            fillEnumArray(DragProtocol.class, protocol, _protocol);
        }
    }

    private static native final void gdk_drag_find_window_for_screen(long self, long dragWindow, long screen, int xRoot, int yRoot, long[] destWindow, int[] protocol);

    static final boolean dragMotion(DragContext self, Window destWindow, DragProtocol protocol, int xRoot, int yRoot, DragAction suggestedAction, DragAction possibleActions, int time) {
        boolean result;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (destWindow == null) {
            throw new IllegalArgumentException("destWindow can't be null");
        }

        if (protocol == null) {
            throw new IllegalArgumentException("protocol can't be null");
        }

        if (suggestedAction == null) {
            throw new IllegalArgumentException("suggestedAction can't be null");
        }

        if (possibleActions == null) {
            throw new IllegalArgumentException("possibleActions can't be null");
        }

        synchronized (lock) {
            result = gdk_drag_motion(pointerOf(self), pointerOf(destWindow), numOf(protocol), xRoot, yRoot, numOf(suggestedAction), numOf(possibleActions), time);

            return result;
        }
    }

    private static native final boolean gdk_drag_motion(long self, long destWindow, int protocol, int xRoot, int yRoot, int suggestedAction, int possibleActions, int time);

    static final void dragDrop(DragContext self, int time) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            gdk_drag_drop(pointerOf(self), time);
        }
    }

    private static native final void gdk_drag_drop(long self, int time);

    static final void dragAbort(DragContext self, int time) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            gdk_drag_abort(pointerOf(self), time);
        }
    }

    private static native final void gdk_drag_abort(long self, int time);

    static final boolean dragDropSucceeded(DragContext self) {
        boolean result;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            result = gdk_drag_drop_succeeded(pointerOf(self));

            return result;
        }
    }

    private static native final boolean gdk_drag_drop_succeeded(long self);

    static final void finish(DragContext self, boolean success, boolean del, int time) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            gtk_drag_finish(pointerOf(self), success, del, time);
        }
    }

    private static native final void gtk_drag_finish(long self, boolean success, boolean del, int time);

    static final Widget getSourceWidget(DragContext self) {
        long result;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            result = gtk_drag_get_source_widget(pointerOf(self));

            return (Widget) objectFor(result);
        }
    }

    private static native final long gtk_drag_get_source_widget(long self);

    static final void setIconWidget(DragContext self, Widget widget, int hotX, int hotY) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (widget == null) {
            throw new IllegalArgumentException("widget can't be null");
        }

        synchronized (lock) {
            gtk_drag_set_icon_widget(pointerOf(self), pointerOf(widget), hotX, hotY);
        }
    }

    private static native final void gtk_drag_set_icon_widget(long self, long widget, int hotX, int hotY);

    static final void setIconPixmap(DragContext self, Colormap colormap, Pixmap pixmap, Bitmap mask, int hotX, int hotY) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (colormap == null) {
            throw new IllegalArgumentException("colormap can't be null");
        }

        if (pixmap == null) {
            throw new IllegalArgumentException("pixmap can't be null");
        }

        if (mask == null) {
            throw new IllegalArgumentException("mask can't be null");
        }

        synchronized (lock) {
            gtk_drag_set_icon_pixmap(pointerOf(self), pointerOf(colormap), pointerOf(pixmap), pointerOf(mask), hotX, hotY);
        }
    }

    private static native final void gtk_drag_set_icon_pixmap(long self, long colormap, long pixmap, long mask, int hotX, int hotY);

    static final void setIconPixbuf(DragContext self, Pixbuf pixbuf, int hotX, int hotY) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (pixbuf == null) {
            throw new IllegalArgumentException("pixbuf can't be null");
        }

        synchronized (lock) {
            gtk_drag_set_icon_pixbuf(pointerOf(self), pointerOf(pixbuf), hotX, hotY);
        }
    }

    private static native final void gtk_drag_set_icon_pixbuf(long self, long pixbuf, int hotX, int hotY);

    static final void setIconStock(DragContext self, String stockId, int hotX, int hotY) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (stockId == null) {
            throw new IllegalArgumentException("stockId can't be null");
        }

        synchronized (lock) {
            gtk_drag_set_icon_stock(pointerOf(self), stockId, hotX, hotY);
        }
    }

    private static native final void gtk_drag_set_icon_stock(long self, String stockId, int hotX, int hotY);

    static final void setIconDefault(DragContext self) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            gtk_drag_set_icon_default(pointerOf(self));
        }
    }

    private static native final void gtk_drag_set_icon_default(long self);

    static final void setIconName(DragContext self, String iconName, int hotX, int hotY) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (iconName == null) {
            throw new IllegalArgumentException("iconName can't be null");
        }

        synchronized (lock) {
            gtk_drag_set_icon_name(pointerOf(self), iconName, hotX, hotY);
        }
    }

    private static native final void gtk_drag_set_icon_name(long self, String iconName, int hotX, int hotY);
}

