/*
 * GdkEvent.java
 *
 * Copyright (c) 2006-2009 Operational Dynamics Consulting Pty Ltd, and Others
 *
 * The code in this file, and the program it is a part of, are made available
 * to you by the authors under the terms of the "GNU General Public Licence,
 * version 2" plus the "Classpath Exception" (you may link to this code as a
 * library into other programs provided you don't make a derivation of it).
 * See the LICENCE file for the terms governing usage and redistribution.
 *
 *                      THIS FILE IS GENERATED CODE!
 *
 * To modify its contents or behaviour, either update the generation program,
 * change the information in the source defs file, or implement an override for
 * this class.
 */
package org.gnome.gdk;

import org.gnome.gdk.AxisUse;
import org.gnome.gdk.Event;
import org.gnome.gdk.EventType;
import org.gnome.gdk.ModifierType;
import org.gnome.gdk.Plumbing;
import org.gnome.gdk.Screen;

final class GdkEvent extends Plumbing
{
    private GdkEvent() {}

    static final boolean sendClientMessage(Event self, long winid) {
        boolean result;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            result = gdk_event_send_client_message(pointerOf(self), winid);

            return result;
        }
    }

    private static native final boolean gdk_event_send_client_message(long self, long winid);

    static final void sendClientmessageToall(Event self) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            gdk_event_send_clientmessage_toall(pointerOf(self));
        }
    }

    private static native final void gdk_event_send_clientmessage_toall(long self);

    static final void put(Event self) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            gdk_event_put(pointerOf(self));
        }
    }

    private static native final void gdk_event_put(long self);

    static final long createEvent(EventType type) {
        long result;

        if (type == null) {
            throw new IllegalArgumentException("type can't be null");
        }

        synchronized (lock) {
            result = gdk_event_new(numOf(type));

            return result;
        }
    }

    private static native final long gdk_event_new(int type);

    static final Event copy(Event self) {
        long result;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            result = gdk_event_copy(pointerOf(self));

            return (Event) boxedFor(Event.class, result);
        }
    }

    private static native final long gdk_event_copy(long self);

    static final void free(Event self) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            gdk_event_free(pointerOf(self));
        }
    }

    private static native final void gdk_event_free(long self);

    static final int getTime(Event self) {
        int result;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            result = gdk_event_get_time(pointerOf(self));

            return result;
        }
    }

    private static native final int gdk_event_get_time(long self);

    static final boolean getState(Event self, ModifierType[] state) {
        boolean result;
        int[] _state;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (state == null) {
            throw new IllegalArgumentException("state can't be null");
        }

        _state = numsOf(state);

        synchronized (lock) {
            result = gdk_event_get_state(pointerOf(self), _state);
            fillFlagArray(ModifierType.class, state, _state);

            return result;
        }
    }

    private static native final boolean gdk_event_get_state(long self, int[] state);

    static final boolean getCoords(Event self, double[] xWin, double[] yWin) {
        boolean result;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (xWin == null) {
            throw new IllegalArgumentException("xWin can't be null");
        }

        if (yWin == null) {
            throw new IllegalArgumentException("yWin can't be null");
        }

        synchronized (lock) {
            result = gdk_event_get_coords(pointerOf(self), xWin, yWin);

            return result;
        }
    }

    private static native final boolean gdk_event_get_coords(long self, double[] xWin, double[] yWin);

    static final boolean getRootCoords(Event self, double[] xRoot, double[] yRoot) {
        boolean result;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (xRoot == null) {
            throw new IllegalArgumentException("xRoot can't be null");
        }

        if (yRoot == null) {
            throw new IllegalArgumentException("yRoot can't be null");
        }

        synchronized (lock) {
            result = gdk_event_get_root_coords(pointerOf(self), xRoot, yRoot);

            return result;
        }
    }

    private static native final boolean gdk_event_get_root_coords(long self, double[] xRoot, double[] yRoot);

    static final boolean getAxis(Event self, AxisUse axisUse, double[] value) {
        boolean result;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (axisUse == null) {
            throw new IllegalArgumentException("axisUse can't be null");
        }

        if (value == null) {
            throw new IllegalArgumentException("value can't be null");
        }

        synchronized (lock) {
            result = gdk_event_get_axis(pointerOf(self), numOf(axisUse), value);

            return result;
        }
    }

    private static native final boolean gdk_event_get_axis(long self, int axisUse, double[] value);

    static final void setScreen(Event self, Screen screen) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (screen == null) {
            throw new IllegalArgumentException("screen can't be null");
        }

        synchronized (lock) {
            gdk_event_set_screen(pointerOf(self), pointerOf(screen));
        }
    }

    private static native final void gdk_event_set_screen(long self, long screen);

    static final Screen getScreen(Event self) {
        long result;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            result = gdk_event_get_screen(pointerOf(self));

            return (Screen) objectFor(result);
        }
    }

    private static native final long gdk_event_get_screen(long self);
}

