/*
 * NotifyNotification.java
 *
 * Copyright (c) 2006-2009 Operational Dynamics Consulting Pty Ltd, and Others
 *
 * The code in this file, and the program it is a part of, are made available
 * to you by the authors under the terms of the "GNU General Public Licence,
 * version 2" plus the "Classpath Exception" (you may link to this code as a
 * library into other programs provided you don't make a derivation of it).
 * See the LICENCE file for the terms governing usage and redistribution.
 *
 *                      THIS FILE IS GENERATED CODE!
 *
 * To modify its contents or behaviour, either update the generation program,
 * change the information in the source defs file, or implement an override for
 * this class.
 */
package org.gnome.notify;

import org.freedesktop.bindings.BlacklistedMethodError;
import org.freedesktop.bindings.FIXME;
import org.gnome.gdk.Pixbuf;
import org.gnome.gdk.Screen;
import org.gnome.glib.GlibException;
import org.gnome.glib.Signal;
import org.gnome.gtk.StatusIcon;
import org.gnome.gtk.Widget;
import org.gnome.notify.Notification;
import org.gnome.notify.Plumbing;
import org.gnome.notify.Urgency;

final class NotifyNotification extends Plumbing
{
    private NotifyNotification() {}

    static final long createNotification(String summary, String body, String icon, Widget attach) {
        long result;

        if (summary == null) {
            throw new IllegalArgumentException("summary can't be null");
        }

        synchronized (lock) {
            result = notify_notification_new(summary, body, icon, pointerOf(attach));

            return result;
        }
    }

    private static native final long notify_notification_new(String summary, String body, String icon, long attach);

    static final long createNotificationWithStatusIcon(String summary, String body, String icon, StatusIcon statusIcon) {
        long result;

        if (summary == null) {
            throw new IllegalArgumentException("summary can't be null");
        }

        if (statusIcon == null) {
            throw new IllegalArgumentException("statusIcon can't be null");
        }

        synchronized (lock) {
            result = notify_notification_new_with_status_icon(summary, body, icon, pointerOf(statusIcon));

            return result;
        }
    }

    private static native final long notify_notification_new_with_status_icon(String summary, String body, String icon, long statusIcon);

    static final boolean update(Notification self, String summary, String body, String icon) {
        boolean result;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (summary == null) {
            throw new IllegalArgumentException("summary can't be null");
        }

        synchronized (lock) {
            result = notify_notification_update(pointerOf(self), summary, body, icon);

            return result;
        }
    }

    private static native final boolean notify_notification_update(long self, String summary, String body, String icon);

    static final void attachToWidget(Notification self, Widget attach) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (attach == null) {
            throw new IllegalArgumentException("attach can't be null");
        }

        synchronized (lock) {
            notify_notification_attach_to_widget(pointerOf(self), pointerOf(attach));
        }
    }

    private static native final void notify_notification_attach_to_widget(long self, long attach);

    static final void attachToStatusIcon(Notification self, StatusIcon statusIcon) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (statusIcon == null) {
            throw new IllegalArgumentException("statusIcon can't be null");
        }

        synchronized (lock) {
            notify_notification_attach_to_status_icon(pointerOf(self), pointerOf(statusIcon));
        }
    }

    private static native final void notify_notification_attach_to_status_icon(long self, long statusIcon);

    static final void setGeometryHints(Notification self, Screen screen, int x, int y) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (screen == null) {
            throw new IllegalArgumentException("screen can't be null");
        }

        synchronized (lock) {
            notify_notification_set_geometry_hints(pointerOf(self), pointerOf(screen), x, y);
        }
    }

    private static native final void notify_notification_set_geometry_hints(long self, long screen, int x, int y);

    static final boolean show(Notification self) throws GlibException {
        boolean result;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            result = notify_notification_show(pointerOf(self));

            return result;
        }
    }

    private static native final boolean notify_notification_show(long self) throws GlibException;

    static final void setTimeout(Notification self, int timeout) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            notify_notification_set_timeout(pointerOf(self), timeout);
        }
    }

    private static native final void notify_notification_set_timeout(long self, int timeout);

    static final void setCategory(Notification self, String category) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (category == null) {
            throw new IllegalArgumentException("category can't be null");
        }

        synchronized (lock) {
            notify_notification_set_category(pointerOf(self), category);
        }
    }

    private static native final void notify_notification_set_category(long self, String category);

    static final void setUrgency(Notification self, Urgency urgency) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (urgency == null) {
            throw new IllegalArgumentException("urgency can't be null");
        }

        synchronized (lock) {
            notify_notification_set_urgency(pointerOf(self), numOf(urgency));
        }
    }

    private static native final void notify_notification_set_urgency(long self, int urgency);

    static final void setIconFromPixbuf(Notification self, Pixbuf icon) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (icon == null) {
            throw new IllegalArgumentException("icon can't be null");
        }

        synchronized (lock) {
            notify_notification_set_icon_from_pixbuf(pointerOf(self), pointerOf(icon));
        }
    }

    private static native final void notify_notification_set_icon_from_pixbuf(long self, long icon);

    static final void setHintInt32(Notification self, String key, int value) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (key == null) {
            throw new IllegalArgumentException("key can't be null");
        }

        synchronized (lock) {
            notify_notification_set_hint_int32(pointerOf(self), key, value);
        }
    }

    private static native final void notify_notification_set_hint_int32(long self, String key, int value);

    static final void setHintDouble(Notification self, String key, double value) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (key == null) {
            throw new IllegalArgumentException("key can't be null");
        }

        synchronized (lock) {
            notify_notification_set_hint_double(pointerOf(self), key, value);
        }
    }

    private static native final void notify_notification_set_hint_double(long self, String key, double value);

    static final void setHintString(Notification self, String key, String value) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (key == null) {
            throw new IllegalArgumentException("key can't be null");
        }

        if (value == null) {
            throw new IllegalArgumentException("value can't be null");
        }

        synchronized (lock) {
            notify_notification_set_hint_string(pointerOf(self), key, value);
        }
    }

    private static native final void notify_notification_set_hint_string(long self, String key, String value);

    static final void setHintByte(Notification self, String key, char value) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        if (key == null) {
            throw new IllegalArgumentException("key can't be null");
        }

        synchronized (lock) {
            notify_notification_set_hint_byte(pointerOf(self), key, value);
        }
    }

    private static native final void notify_notification_set_hint_byte(long self, String key, char value);

    static final void setHintByteArray(Notification self, String key, FIXME value, long len) throws BlacklistedMethodError {
        throw new BlacklistedMethodError("const-guchar*");
    }

    static final void clearHints(Notification self) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            notify_notification_clear_hints(pointerOf(self));
        }
    }

    private static native final void notify_notification_clear_hints(long self);

    static final void addAction(Notification self, String action, String label, FIXME callback, FIXME userData, FIXME freeFunc) throws BlacklistedMethodError {
        throw new BlacklistedMethodError("NotifyActionCallback");
    }

    static final void clearActions(Notification self) {
        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            notify_notification_clear_actions(pointerOf(self));
        }
    }

    private static native final void notify_notification_clear_actions(long self);

    static final boolean close(Notification self) throws GlibException {
        boolean result;

        if (self == null) {
            throw new IllegalArgumentException("self can't be null");
        }

        synchronized (lock) {
            result = notify_notification_close(pointerOf(self));

            return result;
        }
    }

    private static native final boolean notify_notification_close(long self) throws GlibException;

    interface ClosedSignal extends Signal
    {
        void onClosed(Notification source);
    }

    static final void connect(Notification self, NotifyNotification.ClosedSignal handlerInstance, boolean after) {
        connectSignal(self, handlerInstance, NotifyNotification.class, "closed", after);
    }

    protected static final void receiveClosed(Signal handler, long source) {
        ((NotifyNotification.ClosedSignal) handler).onClosed((Notification) objectFor(source));
    }

    interface ActionSignal extends Signal
    {
        void onAction(Notification source, String action);
    }

    static final void connect(Notification self, NotifyNotification.ActionSignal handlerInstance, boolean after) {
        connectSignal(self, handlerInstance, NotifyNotification.class, "action", after);
    }

    protected static final void receiveAction(Signal handler, long source, String action) {
        ((NotifyNotification.ActionSignal) handler).onAction((Notification) objectFor(source), action);
    }
}

